/* utils.h
 * copyright 2009 Bobby Powers, released under the GPLv2+
 * see COPYING for details
 *
 * common useful and interesting defines and declarations
 */
#include <stdint.h>

#ifndef UTILS_H
#define UTILS_H

/* we keep this data in the framebuffer memory, immediately after the drawable
 * pixels */
struct boot_anim_state {
  /* number of frame currently rendered on-screen */
  uint32_t frame_num;

  /* offset (in 2-byte words) of next frame delta from start of framebuffer */
  uint32_t offset;

  /* number of current step in the boot sequence */
  uint32_t step;

  /* total number of steps anticipated in boot sequence */
  uint32_t total_steps;

  /* 1 if we're running, 0 if we've been stopped */
  unsigned char running;

  /* beef cookie to note that the animation started */
  uint32_t cookie;

  /* this is where we keep the diffs */
  unsigned char diff_start[0];
};

struct fbinfo {
  int fd;
  uint8_t bpp;
  union {
    unsigned char *fb;
    uint16_t *fb16;
    uint32_t *fb32;
  };
  struct boot_anim_state *state;
};

#define LINE        0
#define FIRSTCOL    1
#define LEN         2
#define DATA        3

#define FB_SIZE     1200*900	/* pixels */
#define FB_WIDTH    1200
#define FB_HEIGHT   900

/* last frame in the animation series */
#define LAST_FRAME  25

/* extra space (512KB) to mmap, in bytes. this is where
 * we store the diffs and any state information (boot_anim_state). */
#define EXTRA_SPACE 512*1024

/* convert from 565 to ARGB */
#define TOARGB(s)  (0xff070307 | (((s) >> 11) << 19) | ((s & 0x07e0) << 5) | ((s & 0x001f) << 3))

int fb_open(struct fbinfo *info);
void fb_close(struct fbinfo *info);

int chvt_in_userspace (int fd, int num);
void write_image (struct fbinfo *info, const char *file_name);
int dcon_frozen ();
void dcon_freeze (int freeze);

inline int boot_anim_started (struct fbinfo *info);
inline void set_started (struct fbinfo *info);
inline void clear_started (struct fbinfo *info);
inline int is_header (void *p);

#endif /* UTILS_H */

